/*
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This code is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this code; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.  */

/* Copyright (C) 2002 Ian Zimmerman <itz@speakeasy.org> */

static char rcsid[] = "$Id: emacsify.c,v 1.3 2002/09/12 02:37:06 itz Exp $ ";

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string.h>

#include "gim_public.h"

static int drag_x, drag_y;
static char buffer[64];

#define TABLE_SIZE(t) (sizeof(t)/sizeof((t)[0]))

/* itz Mon Mar 23 20:54:54 PST 1998 emacs likes the modifier bits in
  alphabetical order, so I'll use a lookup table instead of a loop; it
  is faster anyways */

static char *s_mod[] = {
  "",                         /* 000 */
  "S-",                       /* 001 */
  "M-",                       /* 002 */
  "M-S-",                     /* 003 */
  "C-",                       /* 004 */
  "C-S-",                     /* 005 */
  "C-M-",                     /* 006 */
  "C-M-S-",                   /* 007 */
  /* idea: maybe we should map AltGr to Emacs Alt instead of Meta? */
  "M-",                       /* 010 */
  "M-S-",                     /* 011 */
  "M-",                       /* 012 */
  "M-S-",                     /* 013 */
  "C-M-",                     /* 014 */
  "C-M-S-",                   /* 015 */
  "C-M-",                     /* 016 */
  "C-M-S-",                   /* 017 */
};
/* itz Mon Mar 23 08:23:14 PST 1998 what emacs calls a `drag' event
     is our `up' event with coordinates different from the `down'
     event.  What gim calls `drag' is just `mouse-movement' to emacs. */

static char *s_type[] = {
  "mouse-movement",
  "mouse-movement",
  "down-mouse-",
  "mouse-",
  0
};

static char *s_button[] = {0,"1","3","2", 0};
static char *s_multi[] = {0, 0, "double-","triple-", 0};
static char s_count[] = "00230";

static long drag_time;

/* itz Mon Mar 23 08:27:53 PST 1998 this flag is needed because even
  if the final coordinates of a drag are identical to the initial
  ones, it is still a drag if there was any movement in between.  Sigh. */
static int drag_flag = 0;

void
emacsify (struct gim_event_t *event)
{
  char count;
  long timestamp;
  int i, j;
  int repeat;
  int button;

  count = '1';
  timestamp = ((short)event->time.tv_sec) * 1000 + (event->time.tv_usec / 1000);
  buffer[0] = 0;

  /* itz Sun Mar 22 19:09:04 PST 1998 Emacs doesn't understand
  modifiers on motion events. */
  if ((event->kind & gim_button_ev) != 0) {
    /* modifiers */
    strcpy(buffer, s_mod[event->modifiers & 017]);
    /* multiple */
    repeat = event->u.b.repeat;
    if (repeat >= 0 && repeat < TABLE_SIZE(s_multi) && s_multi[repeat] != 0) {
      count = s_count[repeat];
      strcat(buffer, s_multi[repeat]);
    } /*if*/
  } /*if*/

  if (event->kind & gim_drag_ev) {
    drag_flag = 1;
  } /*if*/

  /* itz Mon Mar 23 08:26:33 PST 1998 up-event after movement is a drag. */
  if ((event->kind & gim_up_ev) && drag_flag) {
    strcat(buffer, "drag-");
  } /*if*/

  /* type */
  for (i = 0, j = gim_move_ev; s_type[i] != 0; i++, j<<=1)
    if (event->kind & j)
      strcat(buffer, s_type[i]);

  /* button */
  if ((event->kind & gim_button_ev) != 0) {
    button = event->u.b.which;
    if (button >= 0 && button < TABLE_SIZE(s_button) && s_button[button] != 0) {
      strcat(buffer, s_button[button]);
    } /*if*/      
  } /*if*/
  
  /* print the s-representation of the event */
  if ((event->kind & gim_up_ev) != 0 && drag_flag) {
    printf("%s %i %i %ld %c %i %i %ld \n",
           buffer, event->x, event->y, timestamp, count, drag_x, drag_y, drag_time);
  } else if ((event->kind & gim_button_ev) != 0) {
    printf("%s %i %i %ld %c \n",
           buffer, event->x, event->y, timestamp, count);
  } else {
    printf("%s %i %i %ld \n",
           buffer, event->x, event->y, timestamp);
  } /*if*/

  if ((event->kind & gim_down_ev) != 0) {
    drag_x = event->x;
    drag_y = event->y;
    drag_time = timestamp;
    drag_flag = 0;
  } /*if*/

  /* TBA */

}
